import {Metadata, Http, IDictionary, Loader} from '@zeedhi/core';
import {
    Modal,
    ModalService,
    Form,
    Select,
    Grid,
    TextInput,
    AlertService,
    DialogService,
    TreeGrid
} from '@zeedhi/common';
import Store from "@/store";
import { Highlight } from '@zeedhi/zd-highlight-common';
import { CustomAlertService } from '@/Services/CustomAlertService';

export class EstiloCSSController {
    //
    private addModal !: Modal;
    private viewModal !: Modal;
    public viewInputDisable = true;
    private currentRow: any = {};
    protected cssCodeAux: any = "";
    protected cssCodeDebounce: any;
    //
    public reloadGrid() {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        grid.reload();
    }
    //
    public async openAddModal() {
        if (this.addModal !== undefined) {
            await this.addModal?.destroy();
        }
        this.cssCodeAux = "";
        this.addModal = await ModalService.createFromJson('EstiloCSS/AddModal/addModal', true);
        this.addModal.show();
    }

    public async updateLoggedUserInfo() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        this.reloadGrid();
    }

    public async closeAddModal() {
        this.addModal.hide();
        await this.addModal?.destroy();
    }
    //
    public async openViewModal() {
        if (this.viewModal !== undefined) {
            await this.viewModal?.destroy();
        }
        this.viewModal = await ModalService.createFromJson('EstiloCSS/ViewModal/viewModal', true);
        this.viewModal.show();
    }
    //
    public async closeViewModal() {
        this.viewModal.hide();
        await this.viewModal?.destroy();
    }
    //
    public async saveAddModal({component} : any) {
        const form = Metadata.getInstance<Form>('add-form');
        const cssCode = form.getChildInstance<Highlight>('cssCode').code;
        if (!form.validate() || !cssCode.trim()) {
            if (!cssCode.trim()) {
                CustomAlertService.errorAlert("O campo de código é obrigatório e deve ser preenchido");
            }
            return;
        }
        const data = {
            'dataset': [
                    {
                        "__createdLocal": true,
                        "__is_new": true,
                        "changed": true,
                        ...form.value,
                        cssCode
                    }
                ],
                "disableLoader": false,
                'requestType': 'DataSet'
        };
        await Http.post('/style/save', data);
        this.reloadGrid();
        this.closeAddModal();
        CustomAlertService.saveSuccessAlert();
    }
    //
    public async cancelViewModal({component} : any) {
        const form = component.parent.value;
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        const row = grid.datasource.currentRow;
        this.fillForm(form, row);
    }
    //
    public async saveViewModal({component} : any) {
        const form = Metadata.getInstance<Form>('view-form');
        const cssCode = form.getChildInstance<Highlight>('cssCode').code;
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        const row = this.currentRow;
        if (!form.validate() || !cssCode.trim()) {
            if (!cssCode.trim()) {
                CustomAlertService.errorAlert("O campo de código é obrigatório e deve ser preenchido");
            }
            return;
        }
        const data = {
            'dataset': [
                    {
                        "changed": true,
                        "_id": row._id,
                        ...form.value,
                        cssCode
                    }
                ],
                "disableLoader": false,
                'requestType': 'DataSet'
        };
        await Http.post('/style/save', data);
        this.reloadGrid();
        this.closeViewModal();
        CustomAlertService.saveSuccessAlert();
    }
    //
    public async showRowInfo({component, row} : any) {
        if (component.editing) return;
        await this.openViewModal();
        const form = Metadata.getInstance<Form>('view-form').value;
        this.currentRow = row;
        this.fillForm(form, row);
        this.viewInputDisable = false;
        if (row.base === true) {
            this.viewInputDisable = true;
            DialogService.show({
                name: "base-edit-dialog",
                type: "info",
                title: "Atenção",
                text: "Não é possível editar estilos padrões."
            });
        }
    }
    //
    public async fillForm(form:any, row:any) {
        form.name = row.name;
        const formSelect = Metadata.getInstance<Select>('type');
        await formSelect.setValue(row.type);
        form.description = row.description;
        form.cssClass = row.cssClass;
        form.cssCode = row.cssCode;
        const cssCode = Metadata.getInstance<Highlight>('cssCode');
        cssCode.code = row.cssCode || "";
        cssCode.readonly = Boolean(row.base);
    }
    //
    public isRowEditable({ row }: IDictionary<any>) {
        return !row.base;
    }
    //
    get disabled(): boolean {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        return grid.selectedRows.length <= 0;
    }
    //
    public showConfirmDelete({ component, event } : any) {
        event.preventDefault();
        DialogService.show({
            name: 'confirm-delete',
            type: 'warning',
            title: ' ',
            text: 'Deseja remover o(s) registro(s) selecionado(s)?',
            buttons: [
                { name: 'nao', component: 'ZdButton', label: 'Não', events: { click: () => { DialogService.hide(); } }, outline: true },
                { name: 'sim', component: 'ZdButton', label: 'Sim', events: { click: () => { this.deleteRow(); DialogService.hide(); } }},
            ],
        });
    }
    //
    public async deleteRow() {
        const grid = Metadata.getInstance<Grid>('estilo-css-grid');
        for (let i = 0; i < grid.selectedRows.length; i += 1) {
            if (grid.selectedRows[i].base === true) {
                CustomAlertService.errorAlert("Não é possível deletar estilo(s) padrão(ões)!");
                return;
            }
        }
        const data = {
            "dataset": grid.selectedRows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        await Http.post('/style/remove', data);
        CustomAlertService.deleteSuccessAlert();
        this.reloadGrid();
    }
    //
    public get cssCode() {
        return this.cssCodeAux;
    }
    public set cssCode(cssCode: string) {
        this.cssCodeAux = cssCode;
        clearTimeout(this.cssCodeDebounce);
        this.cssCodeDebounce = setTimeout(() => {
            this.setCssClass();
        }, 500);
    }
    //
    public setCssClass() {
        const cssClass = Metadata.getInstance<TextInput>('cssClass');
        const component = Metadata.getInstance<Highlight>('cssCode');
        const cssCode = component.code;
        const classes = cssCode.match(/(\w+)?(\s*>\s*)?(#\w+)?\s*(\.\w+)?\s*{/gm);
        if (classes != null) {
            const newClasses = classes.map(function (classe:string) {
                return classe.trim().replace('.', '').replace('{', '');
            });
            cssClass.value = newClasses.join('');
        } else {
            cssClass.value = "";
        }
    }
    //
    public async inlineEdit({ component }: IDictionary<any>) {
        const rows = component.getEditedRows();
        if (rows.length === 0) {
            return;
        }
        rows.forEach((val: any) => { val.changed = true; });
        const data = {
            'dataset': rows,
            "disableLoader": false,
            'requestType': 'DataSet'
        };
        await Http.post('/style/save', data);
        CustomAlertService.saveSuccessAlert();
        this.reloadGrid();
    }
}
